<template>
    <PageLayout>
        <RouterView v-slot="{ Component }">
            <Component
                :is="Component"
                v-if="Component"
                v-show="!loadingStore.isLoading"
                class="page"
                @page-data-loaded="loadingStore.setIsLoading(false)"
            />
        </RouterView>
    </PageLayout>
</template>

<script setup>
import { useAnalytic } from '@/plugins/AnalyticPlugin/use-analytic';
import { useToast } from '@/plugins/ToastPlugin';

import { markRaw, ref, watch } from 'vue';
import { useRoute, useRouter } from 'vue-router';
import { invoke } from '@vueuse/core';

import { PROMO_QUERY_NAME, ROUTE_NAMES } from '@/router';

import { useAppStore } from '@/store/app.store';
import { useBonusStore } from '@/store/bonus.store';
import { useCatalogStore } from '@/store/catalog.store';
import { useFavoritesStore } from '@/store/favorites.store';
import { useGamesStore } from '@/store/games.store';
import { useLoadingStore } from '@/store/loading.store';
import { useLocalizationStore } from '@/store/localization.store';
import { useThemeStore } from '@/store/theme.store';
import { useUserStore } from '@/store/user.store';
import { useWalletsStore } from '@/store/wallets.store';
import { AUTH_MODAL_QUERY_PARAM, AUTH_MODAL_TABS } from '@/constants/auth-modal';
import { getItem, setItem } from '@/utils/persist-store';

import PageLayout from '@/components/layouts/PageLayout.vue';
import CookiePolicyToast from '@/components/toasts/CookiePolicyToast.vue';

import { NotificationsConnection } from './classes/notifications-connection';
import { useConnection } from './composables/connection';
import { useEmitterEvent } from './composables/use-emitter-event';
import {
    TOAST_COOKIE_POLICY,
    TOAST_ERROR_MESSAGES,
    TOAST_SUCCESS_MESSAGES,
    TOAST_WARNING_MESSAGES,
} from './constants/toast';

const route = useRoute();
const router = useRouter();
const { trackEvent } = useAnalytic();
const { toastShow } = useToast();

const appStore = useAppStore();
const userStore = useUserStore();
const bonusStore = useBonusStore();
const gamesStore = useGamesStore();
const walletsStore = useWalletsStore();
const localizationStore = useLocalizationStore();
const catalogStore = useCatalogStore();
const favoritesStore = useFavoritesStore();
const loadingStore = useLoadingStore();
const themeStore = useThemeStore();

const reconnectionToast = ref(null);
const reconnectionFailureToast = ref(null);

async function fetchData() {
    await appStore.getAppConfig();
    await gamesStore.fetchFilters();
    await userStore.fetchMe();

    await Promise.all([
        localizationStore.fetchLocalizations(),
        catalogStore.fetchCountries(),
        catalogStore.fetchCurrencies(),
        bonusStore.fetchBonusLevels(),
        walletsStore.fetchWallets(),
        favoritesStore.fetchFavorites(),
        bonusStore.fetchCurrentBonusLevel(),
    ]);
}

function onLogout() {
    if (route.meta.redirectNotAuth) router.push({ name: ROUTE_NAMES.MAIN });
    else if (route.meta.requiresAuth) router.push({ query: { [AUTH_MODAL_QUERY_PARAM]: AUTH_MODAL_TABS.login } });
}

function setPromoQuery(value) {
    if (!value) return;
    setItem(PROMO_QUERY_NAME, encodeURIComponent(value));
}

function setPartnerQuery(value) {
    if (!value) return;

    const data = {
        pp_name: route.query.partner,
        ...route.query,
    };

    delete data.partner;

    userStore.getPartners(data);
}

watch(() => route.query.promo, setPromoQuery);
watch(() => route.query.partner, setPartnerQuery);
watch(
    () => appStore?.showCookiesModal,
    (value) => {
        if (value && !getItem(TOAST_COOKIE_POLICY)) {
            toastShow.default(undefined, {
                component: markRaw(CookiePolicyToast),
                duration: 0,
                position: 'BOTTOM_LEFT',
                mobilePosition: 'BOTTOM_CENTER',
            });
        }
    },
);

useConnection();

useEmitterEvent([
    [
        'LOGIN_SUCCESS',
        () => {
            fetchData();
            trackEvent('LOGGED_IN');
        },
    ],
    [
        'LOGOUT_BEFORE',
        () => {
            onLogout();
        },
    ],
    [
        'REGISTER_SUCCESS',
        () => {
            fetchData();
            trackEvent('REGISTERED');
        },
    ],
    [
        'RECONNECTION_START',
        () => {
            if (reconnectionToast.value) return;
            reconnectionToast.value = toastShow.error(TOAST_ERROR_MESSAGES.RECONNECTING_TO_SERVER, { duration: 0 });
        },
    ],
    [
        'RECONNECTION_SUCCESS',
        () => {
            reconnectionToast.value?.();
            reconnectionFailureToast.value?.();
            toastShow.success(TOAST_SUCCESS_MESSAGES.CONNECTION_RESTORED);
        },
    ],
    [
        'RECONNECTION_FAILURE',
        () => {
            reconnectionToast.value?.();
            if (reconnectionFailureToast.value) return;
            reconnectionFailureToast.value = toastShow.error(TOAST_ERROR_MESSAGES.SOMETHING_WENT_WRONG, {
                duration: 0,
            });
        },
    ],
    [
        'ERROR_SOMETHING_WENT_WRONG',
        () => {
            toastShow.error(TOAST_ERROR_MESSAGES.SOMETHING_WENT_WRONG);
        },
    ],
]);

invoke(async () => {
    // Initialize theme before other app initialization
    themeStore.setTheme(themeStore.isDarkTheme);

    loadingStore.setIsLoading(true);
    await fetchData();
    await NotificationsConnection.initialize();

    if (!appStore.screen.breakpoints.isMd) return;
    toastShow.default(TOAST_WARNING_MESSAGES.USE_VPN, { duration: 7000 });
});
</script>
